'use strict';

exports.__esModule = true;
exports.default = styled;
exports.keyframes = keyframes;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _md = require('md5');

var _md2 = _interopRequireDefault(_md);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var STYLED_NAMESPACE = 'styled-namespace';
var e = _react2.default.createElement;
var appended = [];

//styled-components wannabe! take a component
function styled(Component) {
  //then create tagged template literal for that component
  return function createStyled(css) {
    for (var _len = arguments.length, exp = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      exp[_key - 1] = arguments[_key];
    }

    //return a named function, easier to debug in the stack trace
    return function styledComponent(props) {
      var interpolatedCSS = typeof css === 'string' ? ['{', css, '}'].join('') : css.raw.reduce(createRawCSSReducer(exp, props), '');
      //use the component name, else just use styled prefix
      var className = [Component.displayName || Component.name || 'styled',
      //generates a unique identifier based on the css string
      (0, _md2.default)(interpolatedCSS)].join('_');

      renderStyle(className, interpolatedCSS);

      return e(Component, Object.assign({}, props, {
        className: [className, props.className].filter(Boolean).join(' ')
      }));
    };
  };
}

function keyframes(css) {
  var interpolated = css.raw.reduce(createRawCSSReducer());
  var hash = ['keyframes', (0, _md2.default)(interpolated)].join('_');
  var style = ['@keyframes ', hash, '{', interpolated, '}'].join('');
  renderStyle(hash, style);
  return hash;
}

function renderStyle(className, interpolatedCSS) {
  //only append the style in the head if it's not there
  var styleAppended = appended.includes(className);
  if (!styleAppended) {
    //create the style node
    var style = document.createElement('style');
    var cssClassName = '.' + className;
    style.innerHTML = interpolatedCSS.split('\n').map(function (e) {
      return e.trim();
    }).filter(Boolean).join('').split('}').filter(Boolean).map(function (e) {
      return e.includes('{') ? e.includes('@') ? e.split('@').map(function (e, index) {
        if (index) {
          return e.split('{').map(function (e, index, array) {
            switch (true) {
              case index === 0:
                return e + '{ & ';
              case index === array.length - 1:
                return e + '}';
              default:
                return e;
            }
          }).join('{');
        }
        return '& {' + e + '}';
      }).join('@') : '& ' + e : '&{' + e;
    }).join('}').replace(/&/g, cssClassName);
    style.id = className;
    style.className = STYLED_NAMESPACE;

    //append it!
    document.head.insertBefore(style, document.head.firstChild);
    appended.push(className);
  }
}

//take the exp and props variable, and create a reducer function for that.
function createRawCSSReducer(exp, props) {
  //combine the array of css.raw to a string, and invoke any function with props
  return function rawCSSReducer(res, string, i) {
    if (typeof exp[i] === 'function') {
      res += string + exp[i](props);
      return res;
    }
    if (exp[i]) {
      res += string + exp[i];
      return res;
    }
    res += string;
    return res;
  };
}